@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -87,7 +86,8 @@ ok_button_clicked (GtkWidget  *button,
 		time_adjustment = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("adjust_timezone_spinbutton")));
 	}
 
-	task = gth_change_date_task_new (data->file_list,
+	task = gth_change_date_task_new (gth_browser_get_location (data->browser),
+					 data->file_list,
 					 change_fields,
 					 change_type,
 					 date_time,
@@ -153,6 +153,7 @@ dlg_change_date (GthBrowser *browser,
 
 	data->dialog = GET_WIDGET ("change_date_dialog");
 	data->date_selector = gth_time_selector_new ();
+	gth_time_selector_show_time (GTH_TIME_SELECTOR (data->date_selector), TRUE, TRUE);
 
 	datetime = gth_datetime_new ();
 	g_get_current_time (&timeval);
