@@ -39,6 +39,9 @@
 using namespace std;
 
 
+#define INVALID_VALUE "---"
+
+
 /* Some bits of information may be contained in more than one metadata tag.
    The arrays below define the valid tags for a particular piece of
    information, in decreasing order of preference (best one first) */
@@ -57,6 +60,14 @@ const char *_DATE_TAG_NAMES[] = {
 	NULL
 };
 
+const char *_LAST_DATE_TAG_NAMES[] = {
+	"Exif::Image::DateTime",
+	"Xmp::exif::DateTime",
+	"Xmp::xmp::ModifyDate",
+	"Xmp::xmp::MetadataDate",
+	NULL
+};
+
 const char *_ORIGINAL_DATE_TAG_NAMES[] = {
 	"Exif::Photo::DateTimeOriginal",
 	"Xmp::exif::DateTimeOriginal",
@@ -132,11 +143,11 @@ const char *_ORIENTATION_TAG_NAMES[] = {
 };
 
 const char *_DESCRIPTION_TAG_NAMES[] = {
+	"Iptc::Application2::Caption",
+	"Xmp::dc::description",
 	"Exif::Photo::UserComment",
 	"Exif::Image::ImageDescription",
-	"Xmp::dc::description",
 	"Xmp::tiff::ImageDescription",
-	"Iptc::Application2::Caption",
 	"Iptc::Application2::Headline",
 	NULL
 };
@@ -153,9 +164,9 @@ const char *_LOCATION_TAG_NAMES[] = {
 };
 
 const char *_KEYWORDS_TAG_NAMES[] = {
-	"Xmp::dc::subject",
-	"Xmp::iptc::Keywords",
 	"Iptc::Application2::Keywords",
+	"Xmp::iptc::Keywords",
+	"Xmp::dc::subject",
 	NULL
 };
 
@@ -188,14 +199,13 @@ attribute_is_date (const char *key)
 }
 
 
-static void
-set_file_info (GFileInfo  *info,
-	       const char *key,
-	       const char *description,
-	       const char *formatted_value,
-	       const char *raw_value,
-	       const char *category,
-	       const char *type_name)
+static GthMetadata *
+create_metadata (const char *key,
+		 const char *description,
+		 const char *formatted_value,
+		 const char *raw_value,
+		 const char *category,
+		 const char *type_name)
 {
 	char            *attribute;
 	GthMetadataInfo *metadata_info;
@@ -204,7 +214,7 @@ set_file_info (GFileInfo  *info,
 	char            *formatted_value_utf8;
 
 	if (_g_utf8_all_spaces (formatted_value))
-		return;
+		return NULL;
 
 	attribute = exiv2_key_to_attribute (key);
 	description_utf8 = g_locale_to_utf8 (description, -1, NULL, NULL, NULL);
@@ -215,6 +225,8 @@ set_file_info (GFileInfo  *info,
 			formatted_value_utf8 = _g_time_val_strftime (&time_, "%x %X");
 		else
 			formatted_value_utf8 = g_locale_to_utf8 (formatted_value, -1, NULL, NULL, NULL);
+		if (formatted_value_utf8 == NULL)
+			formatted_value_utf8 = g_strdup (INVALID_VALUE);
 	}
 	else {
 		const char *formatted_clean;
@@ -224,6 +236,8 @@ set_file_info (GFileInfo  *info,
 		else
 			formatted_clean = formatted_value;
 		formatted_value_utf8 = g_locale_to_utf8 (formatted_clean, -1, NULL, NULL, NULL);
+		if (formatted_value_utf8 == NULL)
+			formatted_value_utf8 = g_strdup (INVALID_VALUE);
 	}
 
 	metadata_info = gth_main_get_metadata_info (attribute);
@@ -253,35 +267,120 @@ set_file_info (GFileInfo  *info,
 		      "raw", raw_value,
 		      "value-type", type_name,
 		      NULL);
-	g_file_info_set_attribute_object (info, attribute, G_OBJECT (metadata));
 
-	g_object_unref (metadata);
 	g_free (formatted_value_utf8);
 	g_free (description_utf8);
 	g_free (attribute);
+
+	return metadata;
 }
 
 
 static void
-set_attribute_from_tagset (GFileInfo  *info,
-			   const char *attribute,
-			   const char *tagset[])
+set_file_info (GFileInfo  *info,
+	       const char *key,
+	       const char *description,
+	       const char *formatted_value,
+	       const char *raw_value,
+	       const char *category,
+	       const char *type_name)
 {
-	GObject *metadata;
-	int      i;
-	char    *key;
-	char    *description;
-	char    *formatted_value;
-	char    *raw_value;
-	char    *type_name;
+	char        *attribute;
+	GthMetadata *metadata;
 
-	metadata = NULL;
-	for (i = 0; tagset[i] != NULL; i++) {
-		metadata = g_file_info_get_attribute_object (info, tagset[i]);
-		if (metadata != NULL)
-			break;
+	attribute = exiv2_key_to_attribute (key);
+	metadata = create_metadata (key, description, formatted_value, raw_value, category, type_name);
+	if (metadata != NULL) {
+		g_file_info_set_attribute_object (info, attribute, G_OBJECT (metadata));
+		g_object_unref (metadata);
 	}
 
+	g_free (attribute);
+}
+
+
+GHashTable *
+create_metadata_hash (void)
+{
+	return g_hash_table_new_full (g_str_hash,
+				      g_str_equal,
+				      g_free,
+				      g_object_unref);
+}
+
+
+static void
+add_metadata_to_hash (GHashTable  *table,
+		      GthMetadata *metadata)
+{
+	char     *key;
+	gpointer  object;
+
+	if (metadata == NULL)
+		return;
+
+	key = exiv2_key_to_attribute (gth_metadata_get_id (metadata));
+	object = g_hash_table_lookup (table, key);
+	if (object != NULL) {
+		GthStringList *string_list;
+		GList         *list;
+
+		string_list = NULL;
+		if (GTH_IS_METADATA (object)) {
+			string_list = gth_string_list_new (NULL);
+			list = g_list_append (NULL, g_strdup (gth_metadata_get_formatted (GTH_METADATA (object))));
+			gth_string_list_set_list (string_list, list);
+		}
+		else if (GTH_IS_STRING_LIST (object))
+			string_list = GTH_STRING_LIST (g_object_ref (object));
+
+		if (string_list == NULL)
+			return;
+
+		list = gth_string_list_get_list (string_list);
+		list = g_list_append (list, g_strdup (gth_metadata_get_formatted (metadata)));
+		gth_string_list_set_list (string_list, list);
+
+		g_hash_table_replace (table,
+				      g_strdup (key),
+				      g_object_ref (string_list));
+
+		g_object_unref (string_list);
+	}
+	else
+		g_hash_table_insert (table,
+				     g_strdup (key),
+				     g_object_ref (metadata));
+
+	g_free (key);
+}
+
+
+static void
+set_file_info_from_hash (GFileInfo  *info,
+			 GHashTable *table)
+{
+	GHashTableIter iter;
+	gpointer       key;
+	gpointer       value;
+
+	g_hash_table_iter_init (&iter, table);
+	while (g_hash_table_iter_next (&iter, &key, &value))
+		g_file_info_set_attribute_object (info, (char *)key, G_OBJECT (value));
+}
+
+
+static void
+set_attribute_from_metadata (GFileInfo  *info,
+			     const char *attribute,
+			     GObject    *metadata)
+{
+	char *key;
+	char *description;
+	char *formatted_value;
+	char *raw_value;
+	char *type_name;
+
 	if (metadata == NULL)
 		return;
 
@@ -292,6 +391,7 @@ set_attribute_from_tagset (GFileInfo  *info,
 		      "raw", &raw_value,
 		      "value-type", &type_name,
 		      NULL);
+
 	set_file_info (info,
 		       attribute,
 		       description,
@@ -302,16 +402,33 @@ set_attribute_from_tagset (GFileInfo  *info,
 }
 
 
+static void
+set_attribute_from_tagset (GFileInfo  *info,
+			   const char *attribute,
+			   const char *tagset[])
+{
+	GObject *metadata;
+	int      i;
+
+	metadata = NULL;
+	for (i = 0; tagset[i] != NULL; i++) {
+		metadata = g_file_info_get_attribute_object (info, tagset[i]);
+		if (metadata != NULL)
+			break;
+	}
+
+	if (metadata != NULL)
+		set_attribute_from_metadata (info, attribute, metadata);
+}
+
+
 static void
 set_string_list_attribute_from_tagset (GFileInfo  *info,
 				       const char *attribute,
 				       const char *tagset[])
 {
-	GObject        *metadata;
-	int             i;
-	char           *raw;
-	char          **keywords;
-	GthStringList  *string_list;
+	GObject *metadata;
+	int      i;
 
 	metadata = NULL;
 	for (i = 0; tagset[i] != NULL; i++) {
@@ -323,22 +440,53 @@ set_string_list_attribute_from_tagset (GFileInfo  *info,
 	if (metadata == NULL)
 		return;
 
-	g_object_get (metadata, "raw", &raw, NULL);
-	keywords = g_strsplit (raw, ", ", -1);
-	string_list = gth_string_list_new_from_strv (keywords);
-	g_file_info_set_attribute_object (info, attribute, G_OBJECT (string_list));
+	if (GTH_IS_METADATA (metadata)) {
+		char           *raw;
+		char          **keywords;
+		GthStringList  *string_list;
 
-	g_strfreev (keywords);
-	g_free (raw);
+		g_object_get (metadata, "raw", &raw, NULL);
+		keywords = g_strsplit (raw, ",", -1);
+		string_list = gth_string_list_new_from_strv (keywords);
+		g_file_info_set_attribute_object (info, attribute, G_OBJECT (string_list));
+
+		g_strfreev (keywords);
+		g_free (raw);
+	}
+	else if (GTH_IS_STRING_LIST (metadata))
+		g_file_info_set_attribute_object (info, attribute, metadata);
 }
 
 
 static void
 set_attributes_from_tagsets (GFileInfo *info)
 {
-	set_attribute_from_tagset (info, "general::datetime", _DATE_TAG_NAMES);
+	set_attribute_from_tagset (info, "general::datetime", _LAST_DATE_TAG_NAMES);
+	if (g_file_info_get_attribute_object (info, "general::datetime") == NULL)
+		set_attribute_from_tagset (info, "general::datetime", _ORIGINAL_DATE_TAG_NAMES);
+
 	set_attribute_from_tagset (info, "general::description", _DESCRIPTION_TAG_NAMES);
 	set_attribute_from_tagset (info, "general::title", _TITLE_TAG_NAMES);
+
+	/* if iptc::caption and iptc::headline are different use iptc::headline
+	 * to set general::title, if not already set. */
+
+	if (g_file_info_get_attribute_object (info, "general::title") == NULL) {
+		GObject *iptc_caption;
+		GObject *iptc_headline;
+
+		iptc_caption = g_file_info_get_attribute_object (info, "Iptc::Application2::Caption");
+		iptc_headline = g_file_info_get_attribute_object (info, "Iptc::Application2::Headline");
+
+		if ((iptc_caption != NULL)
+		    && (iptc_headline != NULL)
+		    && (g_strcmp0 (gth_metadata_get_raw (GTH_METADATA (iptc_caption)),
+				   gth_metadata_get_raw (GTH_METADATA (iptc_headline))) != 0))
+		{
+			set_attribute_from_metadata (info, "general::title", iptc_headline);
+		}
+	}
+
 	set_attribute_from_tagset (info, "general::location", _LOCATION_TAG_NAMES);
 	set_string_list_attribute_from_tagset (info, "general::tags", _KEYWORDS_TAG_NAMES);
 	set_attribute_from_tagset (info, "Embedded::Photo::DateTimeOriginal", _ORIGINAL_DATE_TAG_NAMES);
@@ -402,6 +550,8 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 
 	Exiv2::IptcData &iptcData = image->iptcData();
 	if (! iptcData.empty()) {
+		GHashTable *table = create_metadata_hash ();
+
 		Exiv2::IptcData::iterator end = iptcData.end();
 		for (Exiv2::IptcData::iterator md = iptcData.begin(); md != end; ++md) {
 			stringstream raw_value;
@@ -413,18 +563,25 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 			else
 				description << md->tagName();
 
-			set_file_info (info,
-				       md->key().c_str(),
-				       description.str().c_str(),
-				       md->print().c_str(),
-				       raw_value.str().c_str(),
-				       "Iptc",
-				       md->typeName());
+			GthMetadata *metadata;
+			metadata = create_metadata (md->key().c_str(),
+						    description.str().c_str(),
+						    md->print().c_str(),
+						    raw_value.str().c_str(),
+						    "Iptc",
+						    md->typeName());
+			add_metadata_to_hash (table, metadata);
+			_g_object_unref (metadata);
 		}
+
+		set_file_info_from_hash (info, table);
+		g_hash_table_unref (table);
 	}
 
 	Exiv2::XmpData &xmpData = image->xmpData();
 	if (! xmpData.empty()) {
+		GHashTable *table = create_metadata_hash ();
+
 		Exiv2::XmpData::iterator end = xmpData.end();
 		for (Exiv2::XmpData::iterator md = xmpData.begin(); md != end; ++md) {
 			stringstream raw_value;
@@ -436,14 +593,19 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 			else
 				description << md->groupName() << "." << md->tagName();
 
-			set_file_info (info,
-				       md->key().c_str(),
-				       description.str().c_str(),
-				       md->print().c_str(),
-				       raw_value.str().c_str(),
-				       "Xmp::Embedded",
-				       md->typeName());
+			GthMetadata *metadata;
+			metadata = create_metadata (md->key().c_str(),
+						    description.str().c_str(),
+						    md->print().c_str(),
+						    raw_value.str().c_str(),
+						    "Xmp::Embedded",
+						    md->typeName());
+			add_metadata_to_hash (table, metadata);
+			_g_object_unref (metadata);
 		}
+
+		set_file_info_from_hash (info, table);
+		g_hash_table_unref (table);
 	}
 
 	set_attributes_from_tagsets (info);
@@ -544,6 +706,8 @@ exiv2_read_sidecar (GFile     *file,
 			return FALSE;
 
 		if (! xmpData.empty()) {
+			GHashTable *table = create_metadata_hash ();
+
 			Exiv2::XmpData::iterator end = xmpData.end();
 			for (Exiv2::XmpData::iterator md = xmpData.begin(); md != end; ++md) {
 				stringstream raw_value;
@@ -555,14 +719,19 @@ exiv2_read_sidecar (GFile     *file,
 				else
 					description << md->groupName() << "." << md->tagName();
 
-				set_file_info (info,
-					       md->key().c_str(),
-					       description.str().c_str(),
-					       md->print().c_str(),
-					       raw_value.str().c_str(),
-					       "Xmp::Sidecar",
-					       md->typeName());
+				GthMetadata *metadata;
+				metadata = create_metadata (md->key().c_str(),
+							    description.str().c_str(),
+							    md->print().c_str(),
+							    raw_value.str().c_str(),
+							    "Xmp::Sidecar",
+							    md->typeName());
+				add_metadata_to_hash (table, metadata);
+				_g_object_unref (metadata);
 			}
+
+			set_file_info_from_hash (info, table);
+			g_hash_table_unref (table);
 		}
 		Exiv2::XmpParser::terminate();
 
@@ -600,20 +769,22 @@ mandatory_string (Exiv2::ExifData &checkdata,
 
 
 const char *
-gth_main_get_metadata_type (GthMetadata *metadata,
-			    const char  *key)
+gth_main_get_metadata_type (gpointer    metadata,
+			    const char *attribute)
 {
 	const char      *value_type;
 	GthMetadataInfo *metadatum_info;
 
-	value_type = gth_metadata_get_value_type (metadata);
-	if (g_strcmp0 (value_type, "Undefined") == 0)
+	if (GTH_IS_METADATA (metadata)) {
+		value_type = gth_metadata_get_value_type (GTH_METADATA (metadata));
+		if (g_strcmp0 (value_type, "Undefined") == 0)
 			value_type = NULL;
 
-	if (value_type != NULL)
-		return value_type;
+		if (value_type != NULL)
+			return value_type;
+	}
 
-	metadatum_info = gth_main_get_metadata_info (key);
+	metadatum_info = gth_main_get_metadata_info (attribute);
 	if (metadatum_info != NULL)
 		value_type = metadatum_info->type;
 
@@ -655,9 +826,9 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 			 */
 
 			const char *raw_value = gth_metadata_get_raw (metadatum);
-			const char *value_type = gth_main_get_metadata_type (metadatum, key);
+			const char *value_type = gth_main_get_metadata_type (metadatum, attributes[i]);
 
-			if ((raw_value != NULL) && (strcmp (raw_value, "") != 0) &&  (value_type != NULL)) {
+			if ((raw_value != NULL) && (strcmp (raw_value, "") != 0) && (value_type != NULL)) {
 				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (value_type));
 				value->read (raw_value);
 				Exiv2::ExifKey exif_key(key);
@@ -745,19 +916,39 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 	Exiv2::IptcData id;
 	attributes = g_file_info_list_attributes (info, "Iptc");
 	for (i = 0; attributes[i] != NULL; i++) {
-		GthMetadata *metadatum = (GthMetadata *) g_file_info_get_attribute_object (info, attributes[i]);
-		char *key = exiv2_key_from_attribute (attributes[i]);
+		gpointer  metadatum = (GthMetadata *) g_file_info_get_attribute_object (info, attributes[i]);
+		char     *key = exiv2_key_from_attribute (attributes[i]);
 
 		try {
-			const char *raw_value = gth_metadata_get_raw (metadatum);
-			const char *value_type = gth_main_get_metadata_type (metadatum, key);
+			const char *value_type;
 
-			if ((raw_value != NULL) && (strcmp (raw_value, "") != 0) &&  (value_type != NULL)) {
+			value_type = gth_main_get_metadata_type (metadatum, attributes[i]);
+			if (value_type != NULL) {
 				/* See the exif data code above for an explanation. */
 				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (value_type));
-				value->read (raw_value);
 				Exiv2::IptcKey iptc_key(key);
-				id.add (iptc_key, value.get());
+
+				if (GTH_IS_STRING_LIST (metadatum)) {
+					GthStringList *string_list;
+					GList         *scan;
+
+					string_list = GTH_STRING_LIST (metadatum);
+					for (scan = gth_string_list_get_list (string_list); scan; scan = scan->next) {
+						char *single_value = (char *) scan->data;
+
+						value->read (single_value);
+						id.add (iptc_key, value.get());
+					}
+				}
+				else if (GTH_IS_METADATA (metadatum)) {
+					const char *raw_value;
+
+					raw_value = gth_metadata_get_raw (GTH_METADATA (metadatum));
+					if ((raw_value != NULL) && (strcmp (raw_value, "") != 0)) {
+						value->read (raw_value);
+						id.add (iptc_key, value.get());
+					}
+				}
 			}
 		}
 		catch (Exiv2::AnyError& e) {
@@ -774,26 +965,46 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 	Exiv2::XmpData xd;
 	attributes = g_file_info_list_attributes (info, "Xmp");
 	for (i = 0; attributes[i] != NULL; i++) {
-		GthMetadata *metadatum = (GthMetadata *) g_file_info_get_attribute_object (info, attributes[i]);
-		char *key = exiv2_key_from_attribute (attributes[i]);
+		gpointer  metadatum = (GthMetadata *) g_file_info_get_attribute_object (info, attributes[i]);
+		char     *key = exiv2_key_from_attribute (attributes[i]);
 
 		// Remove existing tags of the same type.
 		// Seems to be needed for storing category keywords.
 		// Not exactly sure why!
-		Exiv2::XmpData::iterator iter = xd.findKey (Exiv2::XmpKey (key));
+		/*Exiv2::XmpData::iterator iter = xd.findKey (Exiv2::XmpKey (key));
 		if (iter != xd.end ())
-			xd.erase (iter);
+			xd.erase (iter);*/
 
 		try {
-			const char *raw_value = gth_metadata_get_raw (metadatum);
-			const char *value_type = gth_main_get_metadata_type (metadatum, key);
+			const char *value_type;
 
-			if ((raw_value != NULL) && (strcmp (raw_value, "") != 0) &&  (value_type != NULL)) {
+			value_type = gth_main_get_metadata_type (metadatum, attributes[i]);
+			if (value_type != NULL) {
 				/* See the exif data code above for an explanation. */
 				Exiv2::Value::AutoPtr value = Exiv2::Value::create (Exiv2::TypeInfo::typeId (value_type));
-				value->read (raw_value);
 				Exiv2::XmpKey xmp_key(key);
-				xd.add (xmp_key, value.get());
+
+				if (GTH_IS_STRING_LIST (metadatum)) {
+					GthStringList *string_list;
+					GList         *scan;
+
+					string_list = GTH_STRING_LIST (metadatum);
+					for (scan = gth_string_list_get_list (string_list); scan; scan = scan->next) {
+						char *single_value = (char *) scan->data;
+
+						value->read (single_value);
+						xd.add (xmp_key, value.get());
+					}
+				}
+				else if (GTH_IS_METADATA (metadatum)) {
+					const char *raw_value;
+
+					raw_value = gth_metadata_get_raw (GTH_METADATA (metadatum));
+					if ((raw_value != NULL) && (strcmp (raw_value, "") != 0)) {
+						value->read (raw_value);
+						xd.add (xmp_key, value.get());
+					}
+				}
 			}
 		}
 		catch (Exiv2::AnyError& e) {
@@ -909,63 +1120,81 @@ exiv2_generate_thumbnail (const char *uri,
 		char *path;
 
 		path = g_filename_from_uri (uri, NULL, NULL);
-		if (path != NULL) {
-			Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open (path);
-			image->readMetadata ();
-			Exiv2::ExifThumbC exifThumb (image->exifData ());
-			Exiv2::DataBuf thumb = exifThumb.copy ();
+		if (path == NULL)
+			return NULL;
 
-			if (thumb.pData_ != NULL) {
-				Exiv2::ExifData &ed = image->exifData();
+		Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open (path);
+		image->readMetadata ();
+		Exiv2::ExifThumbC exifThumb (image->exifData ());
+		Exiv2::DataBuf thumb = exifThumb.copy ();
 
-				long orientation = ed["Exif.Image.Orientation"].toLong();
-				long image_width = ed["Exif.Photo.PixelXDimension"].toLong();
-				long image_height = ed["Exif.Photo.PixelYDimension"].toLong();
+		g_free (path);
 
-				if ((orientation == 1) && (image_width > 0) && (image_height > 0)) {
-					GInputStream *stream = g_memory_input_stream_new_from_data (thumb.pData_, thumb.size_, NULL);
-					pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, NULL);
+		if (thumb.pData_ == NULL)
+			return NULL;
 
-					if (pixbuf != NULL) {
-						/* Heuristic to find out-of-date thumbnails: the thumbnail and image aspect ratios must be equal */
+		Exiv2::ExifData &ed = image->exifData();
 
-						double image_ratio = (((double) image_width) / image_height);
-						double thumbnail_ratio = (((double) gdk_pixbuf_get_width (pixbuf)) / gdk_pixbuf_get_height (pixbuf));
-						double ratio_delta = (image_ratio > thumbnail_ratio) ? (image_ratio - thumbnail_ratio) : (thumbnail_ratio - image_ratio);
+		long orientation = ed["Exif.Image.Orientation"].toLong();
+		long image_width = ed["Exif.Photo.PixelXDimension"].toLong();
+		long image_height = ed["Exif.Photo.PixelYDimension"].toLong();
 
-						if (ratio_delta > MAX_RATIO_ERROR_TOLERANCE) {
-							g_object_unref (pixbuf);
-							pixbuf = NULL;
-						}
-						else {
-							/* Save the original image size in the pixbuf options */
+		if ((orientation != 1) || (image_width <= 0) || (image_height <= 0))
+			return NULL;
 
-							char *s = g_strdup_printf ("%ld", image_width);
-							gdk_pixbuf_set_option (pixbuf, "tEXt::Thumb::Image::Width", s);
-							g_object_set_data (G_OBJECT (pixbuf), "gnome-original-width", GINT_TO_POINTER ((int) image_width));
-							g_free (s);
+		GInputStream *stream = g_memory_input_stream_new_from_data (thumb.pData_, thumb.size_, NULL);
+		pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, NULL);
+		g_object_unref (stream);
 
-							s = g_strdup_printf ("%ld", image_height);
-							gdk_pixbuf_set_option (pixbuf, "tEXt::Thumb::Image::Height", s);
-							g_object_set_data (G_OBJECT (pixbuf), "gnome-original-height", GINT_TO_POINTER ((int) image_height));
-							g_free (s);
+		if (pixbuf == NULL)
+			return NULL;
 
-							/* Set the orientation option to correctly rotate the thumbnail
-							 * in gnome_desktop_thumbnail_factory_generate_thumbnail() */
+		/* Heuristic to find out-of-date thumbnails: the thumbnail and image aspect ratios must be equal */
 
-							long  orientation = ed["Exif.Image.Orientation"].toLong();
-							char *orientation_s = g_strdup_printf ("%ld", orientation);
-							gdk_pixbuf_set_option (pixbuf, "orientation", orientation_s);
-							g_free (orientation_s);
-						}
-					}
+		int    pixbuf_width = gdk_pixbuf_get_width (pixbuf);
+		int    pixbuf_height = gdk_pixbuf_get_height (pixbuf);
+		double image_ratio = (((double) image_width) / image_height);
+		double thumbnail_ratio = (((double) pixbuf_width) / pixbuf_height);
+		double ratio_delta = (image_ratio > thumbnail_ratio) ? (image_ratio - thumbnail_ratio) : (thumbnail_ratio - image_ratio);
 
-					g_object_unref (stream);
-				}
-			}
-
-			g_free (path);
+		if ((ratio_delta > MAX_RATIO_ERROR_TOLERANCE) /* the tolerance is used because the reduced image can have a slightly different ratio due to rounding errors */
+		    || (MAX (pixbuf_width, pixbuf_height) < (requested_size / 2))) /* if the embedded image is too small compared to the requested size */
+		{
+			g_object_unref (pixbuf);
+			return NULL;
 		}
+
+		/* Scale the pixbuf to perfectly fit the requested size */
+
+		if (scale_keeping_ratio (&pixbuf_width,
+					 &pixbuf_height,
+					 requested_size,
+					 requested_size,
+					 TRUE))
+		{
+			GdkPixbuf *tmp = pixbuf;
+			pixbuf = gdk_pixbuf_scale_simple (tmp, pixbuf_width, pixbuf_height, GDK_INTERP_BILINEAR);
+			g_object_unref (tmp);
+		}
+
+		/* Save the original image size in the pixbuf options */
+
+		char *s = g_strdup_printf ("%ld", image_width);
+		gdk_pixbuf_set_option (pixbuf, "tEXt::Thumb::Image::Width", s);
+		g_object_set_data (G_OBJECT (pixbuf), "gnome-original-width", GINT_TO_POINTER ((int) image_width));
+		g_free (s);
+
+		s = g_strdup_printf ("%ld", image_height);
+		gdk_pixbuf_set_option (pixbuf, "tEXt::Thumb::Image::Height", s);
+		g_object_set_data (G_OBJECT (pixbuf), "gnome-original-height", GINT_TO_POINTER ((int) image_height));
+		g_free (s);
+
+		/* Set the orientation option to correctly rotate the thumbnail
+		 * in gnome_desktop_thumbnail_factory_generate_thumbnail() */
+
+		char *orientation_s = g_strdup_printf ("%ld", orientation);
+		gdk_pixbuf_set_option (pixbuf, "orientation", orientation_s);
+		g_free (orientation_s);
 	}
 	catch (Exiv2::AnyError& e) {
 	}
