@@ -136,7 +136,7 @@ import_dialog_response_cb (GtkDialog *dialog,
 	case GTK_RESPONSE_DELETE_EVENT:
 	case GTK_RESPONSE_CANCEL:
 		picasa_web_accounts_save_to_file (data->accounts, data->email);
-		gtk_widget_destroy (data->dialog);
+		gth_file_list_cancel (GTH_FILE_LIST (data->file_list), (DataFunc) gtk_widget_destroy, data->dialog);
 		break;
 
 	case GTK_RESPONSE_OK:
@@ -305,14 +305,20 @@ list_albums_ready_cb (GObject      *source_object,
 static void
 get_album_list (DialogData *data)
 {
+	char *user_id;
+
+	gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
+
 	if (data->picasaweb == NULL)
 		data->picasaweb = picasa_web_service_new (data->conn);
-	gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
+	user_id = google_utils_get_user_id_from_email (data->email);
 	picasa_web_service_list_albums (data->picasaweb,
-				        "default",
+					user_id,
 				        data->cancellable,
 				        list_albums_ready_cb,
 				        data);
+
+	g_free (user_id);
 }
 
 
@@ -327,8 +333,10 @@ store_password_done_cb (GnomeKeyringResult result,
 
 
 static void account_properties_dialog (DialogData *data,
-			               const char *email);
-static void challange_account_dialog  (DialogData *data);
+			               const char *email,
+			               GError     *error);
+static void challange_account_dialog  (DialogData *data,
+	               	       	       GError     *error);
 
 
 static void
@@ -342,10 +350,12 @@ connection_ready_cb (GObject      *source_object,
 
 	if (! google_connection_connect_finish (conn, result, &error)) {
 		if (g_error_matches (error, GOOGLE_CONNECTION_ERROR, GOOGLE_CONNECTION_ERROR_CAPTCHA_REQUIRED)) {
-			challange_account_dialog (data);
+			challange_account_dialog (data, error);
+			g_clear_error (&error);
 		}
 		else if (g_error_matches (error, GOOGLE_CONNECTION_ERROR, GOOGLE_CONNECTION_ERROR_BAD_AUTHENTICATION)) {
-			account_properties_dialog (data, data->email);
+			account_properties_dialog (data, data->email, error);
+			g_clear_error (&error);
 		}
 		else {
 			if (data->conn != NULL)
@@ -381,6 +391,7 @@ connection_ready_cb (GObject      *source_object,
 
 
 static void connect_to_server (DialogData *data);
+static void auto_select_account (DialogData *data);
 
 
 static void
@@ -412,6 +423,13 @@ account_properties_dialog_response_cb (GtkDialog *dialog,
 		connect_to_server (data);
 		break;
 
+	case PICASA_ACCOUNT_PROPERTIES_RESPONSE_CHOOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		g_free (data->email);
+		data->email = NULL;
+		auto_select_account (data);
+		break;
+
 	default:
 		break;
 	}
@@ -420,7 +438,8 @@ account_properties_dialog_response_cb (GtkDialog *dialog,
 
 static void
 account_properties_dialog (DialogData *data,
-			   const char *email)
+			   const char *email,
+			   GError     *error)
 {
 	GtkWidget *dialog;
 
@@ -428,6 +447,14 @@ account_properties_dialog (DialogData *data,
 		gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 
 	dialog = picasa_account_properties_dialog_new (email, NULL, NULL);
+	picasa_account_properties_dialog_set_error (PICASA_ACCOUNT_PROPERTIES_DIALOG (dialog), error);
+	if ((error != NULL) && (data->accounts != NULL) && (data->accounts->next != NULL))
+		picasa_account_properties_dialog_can_choose (PICASA_ACCOUNT_PROPERTIES_DIALOG (dialog), TRUE);
+
+	g_signal_connect (dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_true),
+			  NULL);
 	g_signal_connect (dialog,
 			  "response",
 			  G_CALLBACK (account_properties_dialog_response_cb),
@@ -443,9 +470,9 @@ account_properties_dialog (DialogData *data,
 static void
 connect_to_server_step2 (DialogData *data)
 {
-	if (data->password == NULL) {
+	if ((data->password == NULL) || g_str_equal (data->password, "")) {
 		gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
-		account_properties_dialog (data, data->email);
+		account_properties_dialog (data, data->email, NULL);
 	}
 	else {
 		gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
@@ -530,6 +557,13 @@ challange_account_dialog_response_cb (GtkDialog *dialog,
 		connect_to_server (data);
 		break;
 
+	case PICASA_ACCOUNT_PROPERTIES_RESPONSE_CHOOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		g_free (data->email);
+		data->email = NULL;
+		auto_select_account (data);
+		break;
+
 	default:
 		break;
 	}
@@ -537,11 +571,20 @@ challange_account_dialog_response_cb (GtkDialog *dialog,
 
 
 static void
-challange_account_dialog (DialogData *data)
+challange_account_dialog (DialogData *data,
+			  GError     *error)
 {
 	GtkWidget *dialog;
 
 	dialog = picasa_account_properties_dialog_new (data->email, data->password, google_connection_get_challange_url (data->conn));
+	picasa_account_properties_dialog_set_error (PICASA_ACCOUNT_PROPERTIES_DIALOG (dialog), error);
+	if ((error != NULL) && (data->accounts != NULL) && (data->accounts->next != NULL))
+		picasa_account_properties_dialog_can_choose (PICASA_ACCOUNT_PROPERTIES_DIALOG (dialog), TRUE);
+
+	g_signal_connect (dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_true),
+			  NULL);
 	g_signal_connect (dialog,
 			  "response",
 			  G_CALLBACK (challange_account_dialog_response_cb),
@@ -583,7 +626,7 @@ account_chooser_dialog_response_cb (GtkDialog *dialog,
 
 	case PICASA_ACCOUNT_CHOOSER_RESPONSE_NEW:
 		gtk_widget_destroy (GTK_WIDGET (dialog));
-		account_properties_dialog (data, NULL);
+		account_properties_dialog (data, NULL, NULL);
 		break;
 
 	default:
@@ -608,7 +651,13 @@ auto_select_account (DialogData *data)
 		else {
 			GtkWidget *dialog;
 
+			if (data->conn != NULL)
+				gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 			dialog = picasa_account_chooser_dialog_new (data->accounts, data->email);
+			g_signal_connect (dialog,
+					  "delete-event",
+					  G_CALLBACK (gtk_true),
+					  NULL);
 			g_signal_connect (dialog,
 					  "response",
 					  G_CALLBACK (account_chooser_dialog_response_cb),
@@ -621,7 +670,7 @@ auto_select_account (DialogData *data)
 		}
 	}
 	else
-		account_properties_dialog (data, NULL);
+		account_properties_dialog (data, NULL, NULL);
 }
 
 
@@ -666,6 +715,10 @@ edit_accounts_button_clicked_cb (GtkButton *button,
 	GtkWidget  *dialog;
 
 	dialog = picasa_account_manager_dialog_new (data->accounts);
+	g_signal_connect (dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_true),
+			  NULL);
 	g_signal_connect (dialog,
 			  "response",
 			  G_CALLBACK (account_manager_dialog_response_cb),
@@ -696,7 +749,11 @@ account_combobox_changed_cb (GtkComboBox *widget,
 
 	if (g_strcmp0 (email, data->email) != 0) {
 		g_free (data->email);
+		g_free (data->password);
+		g_free (data->challange);
 		data->email = email;
+		data->password = NULL;
+		data->challange = NULL;
 		auto_select_account (data);
 	}
 	else
@@ -807,13 +864,16 @@ album_combobox_changed_cb (GtkComboBox *widget,
 
 
 GdkPixbufAnimation *
-picasa_web_thumbnail_loader (GthFileData  *file_data,
-			     GError      **error,
-			     gpointer      data)
+picasa_web_thumbnail_loader (GthFileData   *file_data,
+			     int            requested_size,
+			     int           *original_width,
+			     int           *original_height,
+			     gpointer       user_data,
+			     GCancellable  *cancellable,
+			     GError       **error)
 {
 	GdkPixbufAnimation *animation = NULL;
-	GthThumbLoader     *thumb_loader = data;
-	int                 requested_size;
+	GthThumbLoader     *thumb_loader = user_data;
 	PicasaWebPhoto     *photo;
 	const char         *uri;
 
@@ -842,7 +902,7 @@ picasa_web_thumbnail_loader (GthFileData  *file_data,
 			GdkPixbuf    *pixbuf;
 
 			stream = g_memory_input_stream_new_from_data (buffer, size, g_free);
-			pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, error);
+			pixbuf = gdk_pixbuf_new_from_stream (stream, cancellable, error);
 			if (pixbuf != NULL) {
 				GdkPixbuf *rotated;
 
@@ -937,11 +997,12 @@ dlg_import_from_picasaweb (GthBrowser *browser)
 
 	data->file_list = gth_file_list_new (GTH_FILE_LIST_TYPE_NORMAL, FALSE);
 	thumb_loader = gth_file_list_get_thumb_loader (GTH_FILE_LIST (data->file_list));
-	gth_thumb_loader_use_cache (thumb_loader, FALSE);
-	gth_thumb_loader_set_loader (thumb_loader, picasa_web_thumbnail_loader);
+	gth_thumb_loader_set_use_cache (thumb_loader, FALSE);
+	gth_thumb_loader_set_loader_func (thumb_loader, picasa_web_thumbnail_loader);
 	gth_file_list_set_thumb_size (GTH_FILE_LIST (data->file_list), PICASA_WEB_THUMB_SIZE_SMALL);
 	gth_file_view_set_spacing (GTH_FILE_VIEW (gth_file_list_get_view (GTH_FILE_LIST (data->file_list))), 0);
 	gth_file_list_enable_thumbs (GTH_FILE_LIST (data->file_list), TRUE);
+	gth_file_list_set_ignore_hidden (GTH_FILE_LIST (data->file_list), TRUE);
 	gth_file_list_set_caption (GTH_FILE_LIST (data->file_list), "none");
 	gth_file_list_set_sort_func (GTH_FILE_LIST (data->file_list), picasa_web_photo_position_func, FALSE);
 	gth_file_list_clear (GTH_FILE_LIST (data->file_list), _("No album selected"));
@@ -968,6 +1029,10 @@ dlg_import_from_picasaweb (GthBrowser *browser)
 			  "destroy",
 			  G_CALLBACK (import_dialog_destroy_cb),
 			  data);
+	g_signal_connect (data->dialog,
+			  "delete-event",
+			  G_CALLBACK (gtk_true),
+			  NULL);
 	g_signal_connect (data->dialog,
 			  "response",
 			  G_CALLBACK (import_dialog_response_cb),
