@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -1758,15 +1757,87 @@ save_template (GthWebExporter   *self,
 }
 
 
+enum {
+	_OPEN_IN_BROWSER_RESPONSE = 1,
+	_OPEN_FOLDER_RESPONSE
+};
+
+
+static void
+success_dialog_response_cb (GtkDialog *dialog,
+			    int        response_id,
+			    gpointer   user_data)
+{
+	GthWebExporter *self = user_data;
+	GdkScreen      *screen;
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	switch (response_id) {
+	case _OPEN_IN_BROWSER_RESPONSE:
+	case _OPEN_FOLDER_RESPONSE:
+		{
+			GFile  *file;
+			char   *url = NULL;
+			GError *error = NULL;
+
+			if (response_id == _OPEN_FOLDER_RESPONSE)
+				file = g_object_ref (self->priv->target_dir);
+			else if (response_id == _OPEN_IN_BROWSER_RESPONSE)
+				file = get_html_index_file (self, 0, self->priv->target_dir);
+			else
+				break;
+
+			url = g_file_get_uri (file);
+			if ((url != NULL) && ! gtk_show_uri (screen, url, 0, &error)) {
+				gth_task_dialog (GTH_TASK (self), TRUE, NULL);
+				_gtk_error_dialog_from_gerror_run (GTK_WINDOW (self->priv->browser), _("Could not show the destination"), &error);
+			}
+
+			g_free (url);
+			g_object_unref (file);
+		}
+		break;
+
+	default:
+		break;
+	}
+
+	gth_task_completed (GTH_TASK (self), self->priv->error);
+}
+
+
 static void
 delete_temp_dir_ready_cb (GError   *error,
 			  gpointer  user_data)
 {
 	GthWebExporter *self = user_data;
+	GtkWidget      *dialog;
 
 	if ((self->priv->error == NULL) && (error != NULL))
 		self->priv->error = g_error_copy (error);
-	gth_task_completed (GTH_TASK (self), self->priv->error);
+
+	if (self->priv->error != NULL) {
+		gth_task_completed (GTH_TASK (self), self->priv->error);
+		return;
+	}
+
+	dialog = _gtk_message_dialog_new (GTK_WINDOW (self->priv->browser),
+					  GTK_DIALOG_MODAL,
+					  GTK_MESSAGE_INFO,
+					  _("The album has been created successfully."),
+					  NULL,
+					  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+					  _("_Open in the Browser"), _OPEN_IN_BROWSER_RESPONSE,
+					  _("_View the destination"), _OPEN_FOLDER_RESPONSE,
+					  NULL);
+	g_signal_connect (dialog,
+			  "response",
+			  G_CALLBACK (success_dialog_response_cb),
+			  self);
+	gth_task_dialog (GTH_TASK (self), TRUE, dialog);
+	gtk_window_present (GTK_WINDOW (dialog));
 }
 
 
@@ -2381,31 +2452,19 @@ copy_current_file (GthWebExporter *self)
 	{
 		gboolean appling_tranformation = FALSE;
 
-		if (gth_main_extension_is_active ("image_rotation")
-		    && g_content_type_equals (gth_file_data_get_mime_type (image_data->file_data), "image/jpeg"))
-		{
-			GthMetadata *metadata;
+		if (gth_main_extension_is_active ("image_rotation")) {
+			GthFileData *file_data;
 
-			metadata = (GthMetadata *) g_file_info_get_attribute_object (image_data->file_data->info, "Embedded::Image::Orientation");
-			if (metadata != NULL) {
-				const char *value;
+			file_data = gth_file_data_new (destination, image_data->file_data->info);
+			apply_transformation_async (file_data,
+						    GTH_TRANSFORM_NONE,
+						    JPEG_MCU_ACTION_TRIM,
+						    gth_task_get_cancellable (GTH_TASK (self)),
+						    transformation_ready_cb,
+						    self);
+			appling_tranformation = TRUE;
 
-				value = gth_metadata_get_raw (metadata);
-				if (value != NULL) {
-					GthTransform transform;
-
-					transform = (GthTransform) strtol (value, (char **) NULL, 10);
-					if (transform != 1) {
-						apply_transformation_async (image_data->file_data,
-									    transform,
-									    JPEG_MCU_ACTION_TRIM,
-									    gth_task_get_cancellable (GTH_TASK (self)),
-									    transformation_ready_cb,
-									    self);
-						appling_tranformation = TRUE;
-					}
-				}
-			}
+			g_object_unref (file_data);
 		}
 
 		if (! appling_tranformation)
@@ -2495,16 +2554,35 @@ create_squared_thumbnail (GdkPixbuf     *p,
 }
 
 
-static void
-image_loader_ready_cb (GthImageLoader *iloader,
-		       GError         *error,
-		       gpointer        data)
+static int
+image_data_cmp (gconstpointer a,
+		gconstpointer b,
+		gpointer      user_data)
 {
-	GthWebExporter *self = data;
+	GthWebExporter *self = user_data;
+	ImageData      *idata_a = (ImageData *) a;
+	ImageData      *idata_b = (ImageData *) b;
+
+	return self->priv->sort_type->cmp_func (idata_a->file_data, idata_b->file_data);
+}
+
+
+static void
+image_loader_ready_cb (GObject      *source_object,
+                       GAsyncResult *result,
+                       gpointer      user_data)
+{
+	GthWebExporter *self = user_data;
 	ImageData      *idata;
 	GdkPixbuf      *pixbuf;
 
-	if (error != NULL) {
+	if (! gth_image_loader_load_image_finish (GTH_IMAGE_LOADER (source_object),
+						  result,
+						  &pixbuf,
+						  NULL,
+						  NULL,
+						  NULL))
+	{
 		load_next_file (self);
 		return;
 	}
@@ -2513,7 +2591,7 @@ image_loader_ready_cb (GthImageLoader *iloader,
 
 	/* image */
 
-	idata->image = pixbuf = g_object_ref (gth_image_loader_get_pixbuf (iloader));
+	idata->image = g_object_ref (pixbuf);
 	if (self->priv->copy_images && self->priv->resize_images) {
 		int w = gdk_pixbuf_get_width (pixbuf);
 		int h = gdk_pixbuf_get_height (pixbuf);
@@ -2535,7 +2613,7 @@ image_loader_ready_cb (GthImageLoader *iloader,
 
 	/* preview */
 
-	idata->preview = pixbuf = g_object_ref (gth_image_loader_get_pixbuf (iloader));
+	idata->preview = g_object_ref (pixbuf);
 	if ((self->priv->preview_max_width > 0) && (self->priv->preview_max_height > 0)) {
 		int w = gdk_pixbuf_get_width (pixbuf);
 		int h = gdk_pixbuf_get_height (pixbuf);
@@ -2567,7 +2645,7 @@ image_loader_ready_cb (GthImageLoader *iloader,
 
 	/* thumbnail. */
 
-	idata->thumb = pixbuf = gth_image_loader_get_pixbuf (iloader);
+	idata->thumb = g_object_ref (pixbuf);
 	g_object_ref (idata->thumb);
 
 	if ((self->priv->thumb_width > 0) && (self->priv->thumb_height > 0)) {
@@ -2632,19 +2710,8 @@ image_loader_ready_cb (GthImageLoader *iloader,
 	}
 	else
 		self->priv->saving_timeout = g_idle_add (save_image_preview, self);
-}
 
-
-static int
-image_data_cmp (gconstpointer a,
-		gconstpointer b,
-		gpointer      user_data)
-{
-	GthWebExporter *self = user_data;
-	ImageData      *idata_a = (ImageData *) a;
-	ImageData      *idata_b = (ImageData *) b;
-
-	return self->priv->sort_type->cmp_func (idata_a->file_data, idata_b->file_data);
+	g_object_unref (pixbuf);
 }
 
 
@@ -2668,8 +2735,14 @@ load_current_file (GthWebExporter *self)
 			   g_file_info_get_display_name (file_data->info),
 			   FALSE,
 			   (double) (self->priv->image + 1) / (self->priv->n_images + 1));
-	gth_image_loader_set_file_data (self->priv->iloader, file_data);
-	gth_image_loader_load (self->priv->iloader);
+
+	gth_image_loader_load (self->priv->iloader,
+			       file_data,
+			       -1,
+			       G_PRIORITY_DEFAULT,
+			       gth_task_get_cancellable (GTH_TASK (self)),
+			       image_loader_ready_cb,
+			       self);
 }
 
 
@@ -3012,8 +3085,7 @@ gth_web_exporter_exec (GthTask *task)
 		g_string_append (required_attributes, self->priv->sort_type->required_attributes);
 	}
 	_g_query_all_metadata_async (self->priv->gfile_list,
-				     FALSE,
-				     TRUE,
+				     GTH_LIST_DEFAULT,
 				     required_attributes->str,
 				     gth_task_get_cancellable (GTH_TASK (self)),
 				     file_list_info_ready_cb,
@@ -3128,13 +3200,7 @@ gth_web_exporter_init (GthWebExporter *self)
 	self->priv->file_list = NULL;
 	self->priv->tmp_dir = NULL;
 	self->priv->interrupted = FALSE;
-
-	self->priv->iloader = gth_image_loader_new (FALSE);
-	g_signal_connect (G_OBJECT (self->priv->iloader),
-			  "ready",
-			  G_CALLBACK (image_loader_ready_cb),
-			  self);
-
+	self->priv->iloader = gth_image_loader_new (NULL, NULL);
 	self->priv->error = NULL;
 }
 
