@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -45,6 +44,13 @@ enum {
 };
 
 
+/* Signals */
+enum {
+	LIST_COLLAPSED,
+	LAST_SIGNAL
+};
+
+
 typedef struct {
 	char     *name;
 	gboolean  used;
@@ -74,6 +80,7 @@ struct _GthTagsEntryPrivate {
 
 
 static gpointer parent_class = NULL;
+static guint signals[LAST_SIGNAL] = { 0 };
 
 
 static void
@@ -117,6 +124,18 @@ gth_tags_entry_class_init (GthTagsEntryClass *klass)
 
 	widget_class = (GtkWidgetClass *) klass;
 	widget_class->grab_focus = gth_tags_entry_grab_focus;
+
+	/* signals */
+
+	signals[LIST_COLLAPSED] =
+		g_signal_new ("list-collapsed",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GthTagsEntryClass, list_collapsed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 }
 
 
@@ -129,7 +148,6 @@ get_tag_limits (GthTagsEntry  *self,
 	const char *cursor_start;
 	const char *tag_start;
 	const char *tag_end;
-	const char *tmp_tag_end;
 
 	text = gtk_entry_get_text (GTK_ENTRY (self->priv->entry));
 	cursor_start = g_utf8_offset_to_pointer (text, gtk_editable_get_position (GTK_EDITABLE (self->priv->entry)));
@@ -155,10 +173,6 @@ get_tag_limits (GthTagsEntry  *self,
 	while ((tag_start != tag_end) && g_unichar_isspace (g_utf8_get_char (tag_start)))
 		tag_start = g_utf8_next_char (tag_start);
 
-	tmp_tag_end = g_utf8_strrchr (tag_start, tag_end - tag_start, ' ');
-	if (tmp_tag_end != NULL)
-		tag_end = tmp_tag_end;
-
 	*tag_start_p = tag_start;
 	*tag_end_p = tag_end;
 }
@@ -623,19 +637,7 @@ static void
 tag_list_unmap_cb (GtkWidget    *widget,
 		   GthTagsEntry *self)
 {
-	GtkWidget   *toplevel;
-        GdkGeometry  geometry;
-
-        toplevel = gtk_widget_get_toplevel (widget);
-        if (! GTK_WIDGET_TOPLEVEL (toplevel))
-        	return;
-
-        geometry.max_height = -1;
-        geometry.max_width = G_MAXINT;
-        gtk_window_set_geometry_hints (GTK_WINDOW (toplevel),
-				       toplevel,
-                                       &geometry,
-                                       GDK_HINT_MAX_SIZE);
+        g_signal_emit (self, signals[LIST_COLLAPSED], 0);
 }
 
 
@@ -672,7 +674,7 @@ gth_tags_entry_instance_init (GthTagsEntry *self)
 	GtkTreeViewColumn *column;
 	GtkCellRenderer   *renderer;
 
-	GTK_WIDGET_SET_FLAGS (self, GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus (GTK_WIDGET (self), TRUE);
 
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_TAGS_ENTRY, GthTagsEntryPrivate);
 	self->priv->expanded_list.last_used = g_new0 (char *, 1);
@@ -842,7 +844,7 @@ gth_tags_entry_get_tags (GthTagsEntry *self,
 	for (i = 0, j = 0; all_tags[i] != NULL; i++) {
 		all_tags[i] = g_strstrip (all_tags[i]);
 		if (all_tags[i][0] != '\0') {
-			tags[j] = g_strdup (all_tags[i]);
+			tags[j] = g_strdup (g_strstrip (all_tags[i]));
 			if (update_globals)
 				gth_tags_file_add (tags_file, tags[j]);
 			j++;
